/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */


#include "QE65000Controller.h"

namespace spectrometer {

const string QE65000Controller::TYPE = "spectro";

QE65000Controller::QE65000Controller(const std::string& name) : ExperimentController(name), controller::Stoppable(this) {

	//Up to now only one property
	integrationTime.init(this, SAVE, "integration_time");
	xData.init(this, SAVE, "x_data");
	yData.init(this, SAVE, "y_data");

	spectroDriver.init(this, "driver");

}


QE65000Controller::~QE65000Controller() {
}

void QE65000Controller::postConfiguration() {
	//Pass info to the driver
	registerRefresher(integrationTime, &QE65000Controller::refreshIntegrationTime, this);

	//Get info from driver
	registerUpdater(spectroDriver->yData, &QE65000Controller::updateYData, this);
	registerUpdater(spectroDriver->integrationTime, &QE65000Controller::updateIntegrationTime, this);

	//need to register the progression to see it??
	registerProgression(spectroDriver, &QE65000Controller::updateProgression, this);

}

void QE65000Controller::refreshIntegrationTime(int32 time) {
	cout << "integration time with a new value " << time << endl;

	spectroDriver->integrationTime = time;
}

void QE65000Controller::updateYData() {
	yData.update(spectroDriver->yData());
	yData.setSize(spectroDriver->yData.getSize());

}

void QE65000Controller::updateIntegrationTime() {
	cout << "testing updater!" << spectroDriver->integrationTime() << endl;
}

void QE65000Controller::updateProgression() {

	int32 progression = 0;
	commandProgression = progression;
}

void QE65000Controller::start() {

	commandProgression = 0;

	spectroDriver.execute("start", true); //ask Yannick that!

	//x data
	int32 xSize = spectroDriver->xData.getSize();
	xData.update(spectroDriver->xData());
	xData.setSize(xSize);

	//y data
	int32 ySize = spectroDriver->yData.getSize();
	yData.update(spectroDriver->yData());
	yData.setSize(ySize);

	// first delete current array
//	delete [] xData();
//	float64 * xDataArray = new float64[size];
//	for (int32 i = 0; i < size; i++) {
//		xDataArray[i] = (float64)i;
//	}
//	xData.update(xDataArray);

	commandProgression = 99;
}

void QE65000Controller::stop() {
	spectroDriver.execute("stop", true);
	commandProgression = 100;
}


}




