/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "QE65000Driver.h"
#include "RealQE65000Driver.h"
#include "PerfectQE65000Driver.h"
#include "QE65000State.h"
#include "drivers/global/DriversCommands.h"

namespace oceanoptics {

const std::string QE65000Driver::TYPE = "QE65000";

QE65000Driver::QE65000Driver(const std::string& name) : DeviceDriver(name) {

	registerParentFunction(RS232_FUNCTION); //need to specify parent function

	//Register states --> up to now only RealState
	registerStates(new RealQE65000Driver(this), new PerfectQE65000Driver(this), new PerfectQE65000Driver(this));

	//Init Properties
	integrationTime.init(this, SAVE, "integration_time");
	xData.init(this, SAVE, "x_data");
	yData.init(this, SAVE, "y_data");

	//Init command list
	//initCommand(driver::READ_COMMAND);

	//Init functions
	registerFunction(NONE_FUNCTION);

	//Set type of device (leaf because it will depend on USA49)
	updateValue(DEVICE_TYPE_DEVICE_CONTAINER,std::string(LEAF_DEVICE_TYPE_DEVICE_CONTAINER));

	//Register the Spy and Observer commands necessary to do the updates.
	//registerSpyCommand(driver::READ_COMMAND, 1);
	registerSpyCommand(driver::STATUS_COMMAND, 1);
	//registerObserverCommand(driver::READ_COMMAND, 20);
	registerObserverCommand(driver::STATUS_COMMAND, 20);

}

QE65000Driver::~QE65000Driver() {
}

void QE65000Driver::execute(const std::string& aCommand) {

	//Get the state where to execute the accion
	QE65000State* currentState = dynamic_cast<QE65000State*> (getCurrentState());

	if ((aCommand != COMMAND_STOP_DEVICE_CONTAINER) && (aCommand != COMMAND_START_DEVICE_CONTAINER)) {
		//cout << "sending unknown state" << endl;
		sendProgressEvent(PROGRESSION_UNKNOWNSTATE_DEVICE_CONTAINER);
	}

	if (aCommand == driver::START_COMMAND) {
		//begin reading
		currentState->measure();

	} else if (aCommand == driver::STOP_COMMAND) {
		cout << "Stopping!" << endl;
		currentState->stop();

	}
}

}
