/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include <stdlib.h>

#include "PerfectUsbPVCamDriver.h"
#include "UsbPVCamDriver.h"
#include "UsbPVCamDef.h"
#include "drivers/legacy/def/DetectorDef.h"
#include "drivers/legacy/def/GateDef.h"

// Trace include



using namespace boost;

/*
 * Constructor
 */
PerfectUsbPVCamDriver::PerfectUsbPVCamDriver(UsbPVCamDriver* owner) :
	UsbPVCamState(owner) {
	/* Empty */
}

/*
 * Destructor
 */
PerfectUsbPVCamDriver::~PerfectUsbPVCamDriver() {
	/* Empty */
}

void PerfectUsbPVCamDriver::init() {
}

/*
 * clearvme/daugther/UsbPVCam
 */
void PerfectUsbPVCamDriver::clear() const {
	// Read command
	m_owner->updateValue(GATE_ACTUAL_TIME_DEVICE_CONTAINER, 0.0);
	m_owner->updateValue(GATE_ACTUAL_COUNTER1_DEVICE_CONTAINER, 0);
	m_owner->updateValue(GATE_ACTUAL_COUNTER2_DEVICE_CONTAINER, 0);
	m_owner->updateValue(GATE_ACTUAL_COUNTER3_DEVICE_CONTAINER, 0);
}

/*
 * write
 */
void PerfectUsbPVCamDriver::write() const {
}

/*
 * writeParam
 */
void PerfectUsbPVCamDriver::writeParam() const {
}

/*
 * synchroniseRead
 */
void PerfectUsbPVCamDriver::synchroniseRead() {
	read();
	dynamic_cast<UsbPVCamDriver *> (m_owner)->m_SynchroniseReadTerminated = true;
}

/*
 * read
 */
void PerfectUsbPVCamDriver::read() {
	// Read command
	uint32 size = static_cast<uint32> (any_cast<int32> (
			m_owner->getValue(DETECTOR_DATA_SIZE_DEVICE_CONTAINER)));

	uint32 *tab = reinterpret_cast<uint32*> (any_cast<int32*> (
			m_owner->getValue(DETECTOR_DATA_DEVICE_CONTAINER)));

	int32 sum = 0;
	//  if (m_FirstRead) {
	//    for (uint32 i=0;i<size;i++) {
	//      tab[i] = i + 1;
	//      sum = sum + tab[i];
	//    }
	//    m_FirstRead = false;
	//  } else {
	//    for (uint32 i=0;i<size;i++) {
	//      tab[i] = tab[i] + 1;
	//      sum = sum + tab[i];
	//    }
	//  }
	for (uint32 i = 0; i < size; i++) {
		tab[i] = i;
		sum = sum + tab[i];
	}

}

/*
 * pause 
 */
void PerfectUsbPVCamDriver::pause() {
}

/*
 * continue 
 */
void PerfectUsbPVCamDriver::continu() {
}

/*
 * start
 */
void PerfectUsbPVCamDriver::start() {
	m_owner->updateValue(GATE_ACTUAL_TIME_DEVICE_CONTAINER, any_cast<float64> (
			m_owner->getValue(GATE_WANTED_TIME_DEVICE_CONTAINER)));
}

/*
 * stop
 */
void PerfectUsbPVCamDriver::stop() {
}

/*
 * readStatus
 */
void PerfectUsbPVCamDriver::readStatus() {
	dynamic_cast<UsbPVCamDriver*> (m_owner)->m_StartActivated = false;
	// Status command
	m_owner->updateValue(STATUS_DEVICE_CONTAINER, (int32 ) pvcam::READOUT_COMPLETE);
	// Set category status before ending, synchronism for controller
	m_owner->setCategoryStatus();
	if (dynamic_cast<UsbPVCamDriver *> (m_owner)->m_SynchroniseReadActivated
			== true) {
		if (dynamic_cast<UsbPVCamDriver *> (m_owner)->m_SynchroniseReadTerminated
				== true) {
			dynamic_cast<UsbPVCamDriver *> (m_owner)->m_SynchroniseReadActivated
					= false;
			dynamic_cast<UsbPVCamDriver *> (m_owner)->m_SynchroniseReadTerminated
					= false;
			m_owner->sendProgressEvent(PROGRESSION_END_DEVICE_CONTAINER);
		}
	} else {
		m_owner->sendProgressEvent(PROGRESSION_END_DEVICE_CONTAINER);
	}
}

/*
 * readInfos
 */
void PerfectUsbPVCamDriver::readInfos() {
}
