/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef REALUSBPVCAMDRIVER_H
#define REALUSBPVCAMDRIVER_H

#include "UsbPVCamState.h"
#include "UsbPVCamDef.h"


#include "Utilities/Counter.h"

class UsbPVCamDriver;

/**
 * Real Implementation part of the vme card called UsbPVCam made in Princeton. It needs a board called Men A201.
 */
class RealUsbPVCamDriver: public UsbPVCamState {

public:

	class BadGateMode: public NotOk {
	public:
		BadGateMode(const std::string& type, const std::string& name,
				Publisher* publisher, const std::string& info) :
			NotOk(type, name, publisher, BAD_GATE_MODE_ERROR, info) {
		}
		;
	};

	/**
	 * Constructor.
	 *
	 * @param Link to the class which used the state
	 */
	RealUsbPVCamDriver(UsbPVCamDriver* owner);

	/**
	 * Destructor.
	 */
	virtual ~RealUsbPVCamDriver();

	/**
	 * Initialisation of the device.
	 * Not used yet.
	 */
	virtual void init();

	/**
	 * Clear the gate.
	 * Perform a clear of the device to the clear all counters.
	 */
	virtual void clear() const;

	/**
	 * Write the gate.
	 * Perform a preset of the all counters and registers.
	 */
	virtual void write() const;

	/**
	 * Write the gate.
	 * Perform a preset of the all counters and registers.
	 */
	virtual void writeParam() const;

	/**
	 * Read the gate.
	 * Perform a Read of the gate counter.
	 */
	virtual void read();
	virtual void synchroniseRead();

	/**
	 * Start the gate.
	 * Perform a start of the device to the preset given in the the parameter.
	 */
	virtual void start();

	/**
	 * Continue the gate.
	 * Perform a start of the device to the preset given in the the parameter.
	 */
	virtual void continu();

	/**
	 * Set the gate to mode pause.
	 * Set the gate to the pause mode.
	 */
	virtual void pause();

	/**
	 * Set the gate to mode pause.
	 * Set the gate to the pause mode.
	 */
	virtual void stop();

	/**
	 * Read status of the gate.
	 * Read the status on the gate.
	 */
	virtual void readStatus();

	/**
	 * Read the informations of the hardware.
	 * Read the version, the date,the signatures and local device parameters.
	 */
	virtual void readInfos();

private:

	void print_pv_error();
	int16 m_Handle;
	Counter m_TimeCounter;
};

#endif //REALUSBPVCAMDRIVER_H
