/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef USBPVCAMDRIVER_H
#define USBPVCAMDRIVER_H

#include <Driver.h>
#include "drivers/legacy/category/Gate.h"
#include "drivers/legacy/category/DetectorController.h"

/**
 * Global Implementation of the vme card called UsbPVCam made in Ill. It needs a board called Men A201.
 */
class UsbPVCamDriver: public driver::Gate, public driver::DetectorController {

	friend class RealUsbPVCamDriver;
	friend class PerfectUsbPVCamDriver;
	friend class SimulatedUsbPVCamDriver;

public:
	static const std::string TYPE;


	/**
	 * Constructor.
	 *
	 * @param the type of device driver
	 * @param the name of device driver
	 * @param the category list of device
	 * @param the running mode of ICS software
	 */
	UsbPVCamDriver(const std::string& name);

	/**
	 * Destructor.
	 */
	virtual ~UsbPVCamDriver();

	/**
	 * Update properties value before a set property.
	 */
	virtual void refreshSetValue(std::string propertyName, const boost::any aValue)
			throw (CannotSetValue);

	/**
	 * Execute the command on the device.
	 * The DeviceActionDriver class gives the command to the DeviceDriver.
	 * Each type of device have to interpret its command and call
	 * the write method.
	 *
	 * @param the command to apply on the device
	 */
	virtual void execute(const std::string& aCommand);

	/**
	 * Calculate the progression state.
	 */
	void calculateProgression();

private:

	/**
	 * true for instant command
	 */
	bool m_StartActivated;

	/**
	 * Data table for detector mode
	 */
	int32 m_DataSize;
	int32 m_MaskSize;
	int32* m_Data;
	uint16* m_DataShort;
	int32* m_Mask;

	/**
	 * flag for synchronise read
	 */
	bool m_SynchroniseReadActivated;
	bool m_SynchroniseReadTerminated;

};

#endif //USBPVCAMDRIVER_H
