/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef USBPVCAMSTATE_H
#define USBPVCAMSTATE_H

#include "DeviceContainer/DeviceDriverState.h"
class DeviceDriver;

/**
 * The class is the interface class describes in the pattern state implemented the the UsbPVCam device.
 */
class UsbPVCamState : public DeviceDriverState {

 public:    

  /**
   * Constructor
   *
   * @param the class which used the state
   */
  UsbPVCamState(DeviceDriver* owner) :
    DeviceDriverState(owner)
    {};
  
  /**
   * Destructor
   */
  virtual ~UsbPVCamState() {};

  /**
   * init the gate. 
   * Perform a clear of the device to the clear all counters.
   */
  virtual void init() = 0;
  
  /**
   * Clear the gate. 
   * Perform a clear of the device to the clear all counters.
   */
  virtual void clear() const = 0;
  
  /**
   * Write the gate. 
   * Perform a preset of the all counters and registers.
   */
  virtual void write() const = 0;

  /**
   * Write the gate.
   * Perform a preset of the all counters and registers.
   */
  virtual void writeParam() const = 0;

  /**
   * Read the gate. 
   * Perform a Read of the gate counter.
   */
  virtual void read() = 0;
  virtual void synchroniseRead() = 0;

  /**
   * Start the gate. 
   * Perform a start of the device to the preset given in the the parameter.
   */
  virtual void start() = 0;
  
  /**
   * Continue the gate. 
   * Perform a start of the device to the preset given in the the parameter.
   */
  virtual void continu() = 0;
  
  /**
   * Pause the gate. 
   * Set the gate to the pause mode.
   */
  virtual void pause() = 0;
  
  /**
   * Pause the gate. 
   * Set the gate to the pause mode.
   */
  virtual void stop() = 0;
  
  /**
   * Read status of the gate. 
   * Read the status on the gate.
   */
  virtual void readStatus() = 0;

  /**
   * Read the informations of the hardware. 
   * Read the version, the date,the signatures and local device parameters.
   */
  virtual void readInfos() = 0;
     
};

#endif //USBPVCAMSTATE_H
